"""
This script includes plots of the Gibbs free energy posteriors for the
actual retrieval analyses, where each histogram corresponds to the SNR or ppm
of that particular retrieval run. This script includes results from
Proterozoic Earth retrieved via simulated reflected light and JWST NIRSpec Transit
observations.
"""

import numpy as np
from matplotlib import pyplot as plt

#*~*~*~*~*~*~*Offcial Table of Simulations Gibbs Results/Data*~*~*~**~*~*~*~**~*
#High Protero Earth Composition. Retrieval done with 100K steps, 50K burn-in
#   200 walkers, SNRs 20, 30, and 50, with NIR res 70.
#SNR 20
Gibbs_Data10 = np.genfromtxt('Gibbs_HPE_SNR20.txt',dtype=float)
#SNR 30
Gibbs_Data11 = np.genfromtxt('Gibbs_HPE_SNR30.txt',dtype=float)
#SNR 50
Gibbs_Data12 = np.genfromtxt('Gibbs_HPE_SNR50.txt',dtype=float)

#Avg Protero Earth Composition SNR 20
Gibbs_Data13 = np.genfromtxt('Gibbs_APE_SNR20.txt',dtype=float)
#SNR 30
Gibbs_Data14 = np.genfromtxt('Gibbs_APE_SNR30.txt',dtype=float)
#SNR 50
Gibbs_Data15 = np.genfromtxt('Gibbs_APE_SNR50.txt',dtype=float)

#Low Protero Earth Composition. SNR 20
Gibbs_Data16 = np.genfromtxt('Gibbs_LPE_SNR20.txt',dtype=float)
#SNR 30
Gibbs_Data17 = np.genfromtxt('Gibbs_LPE_SNR30.txt',dtype=float)
#SNR 50
Gibbs_Data18 = np.genfromtxt('Gibbs_LPE_SNR50.txt',dtype=float)

#High Protero Earth Composition, Retrieval done with 200 Walkers, 100K Steps,
#   50K burn-in, 10 ppm, and 20 ppm.
#10 ppm
Gibbs_Data19 = np.genfromtxt('Gibbs_HPE_10ppm.txt',dtype=float)
#5 ppm
Gibbs_Data20 = np.genfromtxt('Gibbs_HPE_5ppm.txt',dtype=float)

#Avg Protero Eath Composition 10ppm
Gibbs_Data22 = np.genfromtxt('Gibbs_APE_10ppm.txt',dtype=float)
#5 ppm
Gibbs_Data23 = np.genfromtxt('Gibbs_APE_5ppm.txt',dtype=float)

#Low Protero Earth Composition 10ppm
Gibbs_Data25 = np.genfromtxt('Gibbs_LPE_10ppm.txt',dtype=float)
#5ppm
Gibbs_Data26 = np.genfromtxt('Gibbs_LPE_5ppm.txt',dtype=float)


#~*~*~*~**~*~*~*~*~*~*~*~*~*~*~*~*~*~*~**~*~*~*~*~*~*~*~*~*~*~*~*~*~*~**~*~*~*~
#Plotting Retrieval Gibbs free energy posteriors for High Protero
#   Earth composition at (20, 30, and 50 SNR)

#Reflected High
# plt.figure(figsize=(11,9))
plt.figure(figsize=(14,6))

#Color variables for High Avg and Low Atmospheric Compositions
c_high = 'green'
c_avg = '#4b0082'
c_low = '#003cff'

#Reflected High
plt.subplot(1,3,1)
plt.hist(np.log10(abs(Gibbs_Data10)),histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_high,density=True)
plt.hist(np.log10(abs(Gibbs_Data11)),histtype='step', alpha=0.3, \
color=c_high,linewidth=2.0,density=True)
plt.hist(np.log10(abs(Gibbs_Data12)),histtype='stepfilled', alpha=0.3, \
facecolor=c_high,density=True)
plt.axvline(np.log10(23.37), linestyle='--', color="black", label='Input',\
zorder=3)
plt.axvline(np.log10(136.34), linestyle='-.', color="firebrick", label='Mars',\
zorder=3)
plt.axvline(np.log10(2325.76), linestyle='-', color="steelblue", label='ME (atm+oc)',\
zorder=3)
plt.axvline(np.log10(1.51), linestyle=':', color="orange", label='ME (atm)')


#Fake Data to make the legend SNR labels grey
plt.hist(np.log10(abs(Gibbs_Data10))+10**6,label='20 SNR',histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= 'grey',density=True)
plt.hist(np.log10(abs(Gibbs_Data11))+10**6,label='30 SNR',histtype='step', alpha=0.3, \
color='grey',linewidth=2.0,density=True)
plt.hist(np.log10(abs(Gibbs_Data12))+10**6,label='50 SNR',histtype='stepfilled', alpha=0.3, \
facecolor='grey',density=True)

# legend = plt.legend(frameon = 1)
# frame = legend.get_frame()
# frame.set_facecolor('green')
plt.legend(loc='upper left',frameon=True,facecolor='white',framealpha=1,edgecolor='white')
plt.text(2.5,1.12,'High',color=c_high)
plt.text(-3.3,1.24,'a',fontweight='bold',fontsize=14)
# plt.xlim([0.0,4.])
plt.xlim([-3.,4.])
# plt.title('Derived from Reflected Light')
plt.xlabel(r'log available Gibbs free energy (J mol$^{-1}$)')
plt.ylabel('Probability density')

#Reflected Avg
plt.subplot(1,3,2)
plt.hist(np.log10(abs(Gibbs_Data13)),label='20 SNR',histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color=c_avg,density=True)
plt.hist(np.log10(abs(Gibbs_Data14)),label='30 SNR',histtype='step', alpha=0.3, \
color=c_avg,linewidth=2.0,density=True)
plt.hist(np.log10(abs(Gibbs_Data15)),label='50 SNR',histtype='stepfilled', alpha=0.3, \
facecolor=c_avg,density=True)
plt.axvline(np.log10(9.41), linestyle='--', color="black", label='Input',\
zorder=3)
plt.axvline(np.log10(136.34), linestyle='-.', color="firebrick", label='Mars',\
zorder=3)
plt.axvline(np.log10(2325.76), linestyle='-', color="steelblue", label='ME (atm+oc)',\
zorder=3)
plt.axvline(np.log10(1.51), linestyle=':', color="orange", label='ME (atm)')
# plt.legend(loc='best',frameon=False)
plt.text(2.5,0.74,'Med.',color=c_avg)
plt.text(-3.3,0.81,'b',fontweight='bold',fontsize=14)
# plt.xlim([0.0,4.])
plt.xlim([-3.,4.])
plt.xlabel(r'log available Gibbs free energy (J mol$^{-1}$)')
plt.ylabel('Probability density')

#Reflected Low
plt.subplot(1,3,3)
plt.hist(np.log10(abs(Gibbs_Data16)),label='20 SNR',histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color=c_low,density=True)
plt.hist(np.log10(abs(Gibbs_Data17)),label='30 SNR',histtype='step', alpha=0.3, \
color=c_low,linewidth=2.0,density=True)
plt.hist(np.log10(abs(Gibbs_Data18)),label='50 SNR',histtype='stepfilled', alpha=0.3, \
facecolor=c_low,density=True)
plt.axvline(np.log10(3.85), linestyle='--', color="black", label='Input',\
zorder=3)
plt.axvline(np.log10(136.34), linestyle='-.', color="firebrick", label='Mars',\
zorder=3)
plt.axvline(np.log10(2325.76), linestyle='-', color="steelblue", label='ME (atm+oc)',\
zorder=3)
plt.axvline(np.log10(1.51), linestyle=':', color="orange", label='ME (atm)')

# plt.legend(loc='best',frameon=False)
plt.text(2.5,0.90,'Low',color=c_low)
plt.text(-3.4,0.99,'c',fontweight='bold',fontsize=14)
plt.xlim([-3.,4.])
# plt.ylim([0.0,0.5])
plt.xlabel(r'log available Gibbs free energy (J mol$^{-1}$)')
plt.ylabel('Probability density')

plt.tight_layout(pad=3)
# plt.savefig('Gibbs_Posterior_PE_Reflected_Retrievals_log.pdf')
# plt.savefig('Protero_Sim_Grid_Full_5_input.svg')
# plt.savefig('Protero_Sim_Grid_Full_5_input.png',dpi=300)
plt.show()
plt.close()

#~*~*~*~*~*~**~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~**~*~*~*
#Publication Supplemental Figure with just transit derived PE Gibbs free energy
#   For all abundance cases at 10ppm and 5ppm
#plt.figure(figsize=(14,6))
fig, ax = plt.subplots(1,3,sharex=False,sharey=False,figsize=(10,5))



#Transit High
plt.subplot(1,3,1)
plt.hist(np.log10(abs(Gibbs_Data19)),histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_high,density=True,label='10 ppm')
plt.hist(np.log10(abs(Gibbs_Data20)),histtype='step', alpha=0.8, \
color=c_high,linewidth=2.0,density=True,label='5 ppm')


plt.axvline(np.log10(584.90), linestyle='--', color="black", label='Input',\
zorder=3)

# plt.legend(loc='upper left',frameon=True,facecolor='white',framealpha=1,edgecolor='white')
plt.legend(loc='upper left',frameon=False)
plt.title('High',color=c_high)
# plt.xlabel(r'log available Gibbs free energy (J mol$^{-1}$)')
# plt.ylabel('Probability density')

fig.text(0.51, 0.03, r'log available Gibbs free energy (J mol$^{-1}$)', ha='center',fontsize=12)
fig.text(0.02, 0.5, 'Probability density', va='center', rotation='vertical',\
          fontsize=12)

#Subpplot Labels
fig.text(0.05,0.91,'a',fontweight='bold',ha='center',fontsize=14)
fig.text(0.39,0.91,'b',fontweight='bold',ha='center',fontsize=14)
fig.text(0.69,0.91,'c',fontweight='bold',ha='center',fontsize=14)

#Transit Avg
plt.subplot(1,3,2)
plt.hist(np.log10(abs(Gibbs_Data22)),histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_avg,density=True)
plt.hist(np.log10(abs(Gibbs_Data23)),histtype='step', alpha=0.8, \
color=c_avg,linewidth=2.0,density=True)

#Recalculate
plt.axvline(np.log10(237.14), linestyle='--', color="black", label='Input',\
zorder=3)

# plt.legend(loc='upper left',frameon=True,facecolor='white',framealpha=1,edgecolor='white')
plt.title('Med.',color=c_avg)
# plt.xlabel(r'log available Gibbs free energy (J mol$^{-1}$)')
# plt.ylabel('Probability density')

#Transit Low
plt.subplot(1,3,3)
plt.hist(np.log10(abs(Gibbs_Data25)),histtype='step', alpha=0.75, \
hatch='//',linewidth=1.0,color= c_low,density=True)
plt.hist(np.log10(abs(Gibbs_Data26)),histtype='step', alpha=0.8, \
color=c_low,linewidth=2.0,density=True)


plt.axvline(np.log10(38.51), linestyle='--', color="black", label='Input',\
zorder=3)

# plt.legend(loc='upper left',frameon=True,facecolor='white',framealpha=1,edgecolor='white')
plt.title('Low',color=c_low)
# plt.xlabel(r'log available Gibbs free energy (J mol$^{-1}$)')
# plt.ylabel('Probability density')


plt.tight_layout(pad=3)
# plt.savefig('Gibbs_Posterior_PE_Transit_Retrievals_SimGrid.pdf')
# plt.savefig('Protero_Sim_Grid_Transit.svg')
# plt.savefig('Protero_Sim_Grid_Transit.png',dpi=300)
plt.show()
plt.close()
