#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Jul 24 18:04:34 2021

@author: avb86
"""

"""
Python module to plot modeled spectra produced from rfast and the spectral
effects of particular species. Pollted here are spectra of simulated Proterozoic
Earth-like planets at high, med. and low abundance scenarios.
"""

import numpy as np
from matplotlib import pyplot as plt
from astropy.io          import ascii
import emcee
import corner

#inset figures
from mpl_toolkits.axes_grid1.inset_locator import inset_axes, zoomed_inset_axes
from mpl_toolkits.axes_grid1.anchored_artists import AnchoredSizeBar
# from mpl_toolkits.axes_grid1.inset_locator import zoomed_inset_axes
# from mpl_toolkits.axes_grid1.inset_locator import mark_inset


#*~*~**~*~*~*~*~**~*~*~**~**~*~*~INPUT DATA*~*~*~*~*~*~*~**~*~*~*~*~**~*

#Reflected High
#   (Use data 39 for high, raw, full spectrum)
#Spectrum Highlighting the effect of CH4
CH4_Heff = ascii.read("earth_HPE_refl_demo_CH4_2times.raw",data_start=1,delimiter='|')
wlgrid_CH4_Heff=CH4_Heff['col2'][:]
GA_CH4_Heff = CH4_Heff['col4'][:]
y_meas_CH4_Heff=CH4_Heff['col5'][:]
#Spectrum Highlighting the effect of O2
O2_Heff = ascii.read("earth_HPE_refl_demo_O2_2times.raw",data_start=1,delimiter='|')
wlgrid_O2_Heff=O2_Heff['col2'][:]
GA_O2_Heff = O2_Heff['col4'][:]
y_meas_O2_Heff=O2_Heff['col5'][:]
#Spectrum Highlighting the effect of CO2
CO2_Heff = ascii.read("earth_HPE_refl_demo_CO2_2times.raw",data_start=1,delimiter='|')
wlgrid_CO2_Heff=CO2_Heff['col2'][:]
GA_CO2_Heff = CO2_Heff['col4'][:]
y_meas_CO2_Heff=CO2_Heff['col5'][:]
#Spectrum Highlighting the effects of O3
O3_Heff = ascii.read("earth_HPE_refl_demo_O3_2times.raw",data_start=1,delimiter='|')
wlgrid_O3_Heff=O3_Heff['col2'][:]
GA_O3_Heff = O3_Heff['col4'][:]
y_meas_O3_Heff=O3_Heff['col5'][:]

#Reflected Avg
#   (Use data 42 for Avg, raw, full spectrum)
#Spectrum Highlighting the effect of CH4
CH4_Aeff = ascii.read("earth_APE_refl_demo_CH4_2times.raw",data_start=1,delimiter='|')
wlgrid_CH4_Aeff=CH4_Aeff['col2'][:]
GA_CH4_Aeff = CH4_Aeff['col4'][:]
y_meas_CH4_Aeff=CH4_Aeff['col5'][:]
#Spectrum Highlighting the effect of O2
O2_Aeff = ascii.read("earth_APE_refl_demo_O2_2times.raw",data_start=1,delimiter='|')
wlgrid_O2_Aeff=O2_Aeff['col2'][:]
GA_O2_Aeff = O2_Aeff['col4'][:]
y_meas_O2_Aeff=O2_Aeff['col5'][:]
#Spectrum Highlighting the effect of CO2
CO2_Aeff = ascii.read("earth_APE_refl_demo_CO2_2times.raw",data_start=1,delimiter='|')
wlgrid_CO2_Aeff=CO2_Aeff['col2'][:]
GA_CO2_Aeff = CO2_Aeff['col4'][:]
y_meas_CO2_Aeff=CO2_Aeff['col5'][:]
#Spectrum Highlighting the effects of O3
O3_Aeff = ascii.read("earth_APE_refl_demo_O3_2times.raw",data_start=1,delimiter='|')
wlgrid_O3_Aeff=O3_Aeff['col2'][:]
GA_O3_Aeff = O3_Aeff['col4'][:]
y_meas_O3_Aeff=O3_Aeff['col5'][:]


#Reflected Low
#   (Use data 45 for Low, raw, full spectrum)
#Spectrum Highlighting the effect of CH4
CH4_Leff = ascii.read("earth_LPE_refl_demo_CH4_2times.raw",data_start=1,delimiter='|')
wlgrid_CH4_Leff=CH4_Leff['col2'][:]
GA_CH4_Leff = CH4_Leff['col4'][:]
y_meas_CH4_Leff=CH4_Leff['col5'][:]
#Spectrum Highlighting the effect of O2
O2_Leff = ascii.read("earth_LPE_refl_demo_O2_2times.raw",data_start=1,delimiter='|')
wlgrid_O2_Leff=O2_Leff['col2'][:]
GA_O2_Leff = O2_Leff['col4'][:]
y_meas_O2_Leff=O2_Leff['col5'][:]
#Spectrum Highlighting the effect of CO2
CO2_Leff = ascii.read("earth_LPE_refl_demo_CO2_2times.raw",data_start=1,delimiter='|')
wlgrid_CO2_Leff=CO2_Leff['col2'][:]
GA_CO2_Leff = CO2_Leff['col4'][:]
y_meas_CO2_Leff=CO2_Leff['col5'][:]
#Spectrum Highlighting the effect of O3
O3_Leff = ascii.read("earth_LPE_refl_demo_O3_2times.raw",data_start=1,delimiter='|')
wlgrid_O3_Leff=O3_Leff['col2'][:]
GA_O3_Leff = O3_Leff['col4'][:]
y_meas_O3_Leff=O3_Leff['col5'][:]

#Full Spectrum Reflected High
Data39 = ascii.read("earth_HPE_refl_demo.raw",data_start=1,delimiter='|')
wlgrid39=Data39['col2'][:]
GA39 = Data39['col4'][:]
y_meas39=Data39['col5'][:]
#Full Spectrum Reflected Avg
Data42 = ascii.read("earth_APE_refl_demo.raw",data_start=1,delimiter='|')
wlgrid42=Data42['col2'][:]
GA42 = Data42['col4'][:]
y_meas42=Data42['col5'][:]
#Full Spectrum Reflected Low
Data45 = ascii.read("earth_LPE_refl_demo.raw",data_start=1,delimiter='|')
wlgrid45=Data45['col2'][:]
GA45 = Data45['col4'][:]
y_meas45=Data45['col5'][:]

#Errorbar Data
#Reflected Low 20 SNR --------
Data27 = ascii.read("PE_Refl_Temp_Retrieval_LowProfile_SNR20_NIR70_1.dat",data_start=1,delimiter='|')
wlgrid27=Data27['col2'][:]
GA27 = Data27['col4'][:]
y_meas27=Data27['col6'][:]
err27=Data27['col7'][:]
#30 SNR
Data28 = ascii.read("PE_Refl_Temp_Retrieval_LowProfile_SNR30_NIR70_1.dat",data_start=1,delimiter='|')
wlgrid28=Data28['col2'][:]
GA28 = Data28['col4'][:]
y_meas28=Data28['col6'][:]
err28=Data28['col7'][:]
#50 SNR
Data29 = ascii.read("PE_Refl_Temp_Retrieval_LowProfile_SNR50_NIR70_1.dat",data_start=1,delimiter='|')
wlgrid29=Data29['col2'][:]
GA29 = Data29['col4'][:]
y_meas29=Data29['col6'][:]
err29=Data29['col7'][:]


#~*~*~**~*~*~*~**~*~**~*~*~*~*~*~*~~*~**~*~*~**~*~*~*~**~*~*~*~*~**~*~*~*~*~*
#Spectral Plotting for Simulated Reflected light Observations


#Color variables for High Avg and Low Atmospheric Compositions
c_high = 'green'
c_avg = '#4b0082'
c_low = '#003cff'

#O2 Abosrption Features
O2_feat_R = [0.762, 1.06, 1.27]
O2_feat_T = [0.762,1.27]
#CH4 Absorption Features
CH4_feat_R = [1.16,1.65]
CH4_feat_T = [1.65,2.31,3.32]
#H2O Absorption Features
H2O_feat_R = [0.600,0.653,0.722,0.823,0.94,1.14,1.38]
#CO2 Absorption Features
CO2_feat_R = [1.6]
#O3 Absorption Features (Hartley-Huggins and Chappuis Bands)
O3_feat_R = [0.256,0.600]


#Making the y-axis tics for the absorption features
#Function to determine y-axis tick marks for key absorption features where wave
# is the center wavelength of the feature, lamd is the whole spectral wavelength
# grid, and dat is the y-axis for for the spectrum
def feature(wave,lamd,dat):
  y_axis_ticks = np.zeros(len(wave))
  for i in range(len(wave)):
    y_axis_ticks[i] = np.interp(wave[i],lamd,dat)
  return y_axis_ticks


O2_feat_y_R = feature(O2_feat_R, wlgrid39, y_meas39)
CH4_feat_y_R = feature(CH4_feat_R, wlgrid39, y_meas39)
O3_feat_y_R = feature(O3_feat_R, wlgrid39, y_meas39)
H2O_feat_y_R = feature(H2O_feat_R, wlgrid39, y_meas39)
CO2_feat_y_R = feature(CO2_feat_R, wlgrid39, y_meas39)

#SPectral Plot of Protero Earth, where species concentrations are varied to
# emphasize their effect on the overall spectrum.
fig, ax = plt.subplots(3,1,sharex=False,sharey=False,figsize=(10,15))

#Reflected High
#High abundance
plt.subplot(3,1,1)

#Full Spectrum
plt.errorbar(wlgrid39,y_meas39,xerr=None,yerr=None,drawstyle='steps-mid',\
color=c_high,alpha=0.7)
plt.fill_between(wlgrid_O3_Heff,y_meas_O3_Heff,y_meas39,facecolor='brown',alpha=0.4,\
step='mid',label=r"O$_3$ $\times 2$")
plt.fill_between(wlgrid_CO2_Heff,y_meas_CO2_Heff,y_meas39,facecolor='RoyalBlue',alpha=0.4,\
step='mid',label=r"CO$_2$ $\times 2$")
plt.fill_between(wlgrid_CH4_Heff,y_meas_CH4_Heff,y_meas39,facecolor='orange',alpha=0.4,\
step='mid',label=r"CH$_4$ $\times 2$",zorder=3)
plt.fill_between(wlgrid_O2_Heff,y_meas_O2_Heff,y_meas39,facecolor='red',alpha=0.4,\
step='mid',label=r"O$_2$ $\times 2$")
plt.xlabel(r'Wavelength (' + u'\u03bc' + 'm)')
plt.ylabel(r'Flux ratio ($F_{\rm p}/F_{\rm s}$)')
plt.legend(loc='best',frameon=False,prop={'size': 12})

H2O_feat_y_R[2] = H2O_feat_y_R[2]*0.90
H2O_feat_y_R[4] = H2O_feat_y_R[4]*-0.5
H2O_feat_y_R[5] = H2O_feat_y_R[3]*0.60
H2O_feat_y_R[6] = H2O_feat_y_R[6]*6


for tick in range(len(H2O_feat_y_R)):
    plt.text(H2O_feat_R[tick],H2O_feat_y_R[tick]*0.79,'H$_2$O',color='black',rotation='vertical')

plt.text(0.9,5.3e-10,'High',color=c_high,fontsize=14)
plt.text(0.02,5.5e-10,'a',fontweight='bold',ha='center',fontsize=14)


#errorbar scaling
plt.vlines(x = 1.15, ymin=5.5e-10-err27[0],ymax=5.5e-10+err27[0],color='grey')
plt.vlines(x = 1.15, ymin=5.01e-10-err28[0],ymax=5.01e-10+err28[0],color='grey')
plt.vlines(x = 1.15, ymin=4.50e-10-err29[0],ymax=4.50e-10+err29[0],color='grey')
plt.text(1.20,5.43e-10,'20 SNR',color='black')
plt.text(1.20,4.95e-10,'30 SNR',color='black')
plt.text(1.20,4.40e-10,'50 SNR',color='black')


#Med abundance
plt.subplot(3,1,2)
plt.errorbar(wlgrid42,y_meas42,xerr=None,yerr=None,drawstyle='steps-mid',\
color=c_avg,alpha=0.5)
plt.fill_between(wlgrid_O3_Aeff,y_meas_O3_Aeff,y_meas42,facecolor='brown',alpha=0.4,\
step='mid',label=r"O$_3$ $\times 2$")
plt.fill_between(wlgrid_CO2_Aeff,y_meas_CO2_Aeff,y_meas42,facecolor='RoyalBlue',alpha=0.4,\
step='mid',label=r"CO$_2$ $\times 2$")
plt.fill_between(wlgrid_CH4_Aeff,y_meas_CH4_Aeff,y_meas42,facecolor='orange',alpha=0.4,\
step='mid',label=r"CH$_4$ $\times 2$",zorder=3)
plt.fill_between(wlgrid_O2_Aeff,y_meas_O2_Aeff,y_meas42,facecolor='red',alpha=0.4,\
step='mid',label=r"O$_2$ $\times 2$")
plt.xlabel(r'Wavelength (' + u'\u03bc' + 'm)')
plt.ylabel(r'Flux ratio ($F_{\rm p}/F_{\rm s}$)')
plt.legend(loc='best',frameon=False,prop={'size': 12})


for tick in range(len(H2O_feat_y_R)):
    plt.text(H2O_feat_R[tick],H2O_feat_y_R[tick]*0.79,'H$_2$O',color='black',rotation='vertical')

plt.text(0.9,5.5e-10,'Med.',color=c_avg,fontsize=14)
#Subpplot Labels
plt.text(0.02,6.0e-10,'b',fontweight='bold',ha='center',fontsize=14)

#Low abundance
plt.subplot(3,1,3)
plt.errorbar(wlgrid45,y_meas45,xerr=None,yerr=None,drawstyle='steps-mid',\
color=c_low,alpha=0.5)
plt.fill_between(wlgrid_O3_Leff,y_meas_O3_Leff,y_meas45,facecolor='brown',alpha=0.4,\
step='mid',label=r"O$_3$ $\times 2$")
plt.fill_between(wlgrid_CO2_Leff,y_meas_CO2_Leff,y_meas45,facecolor='RoyalBlue',alpha=0.4,\
step='mid',label=r"CO$_2$ $\times 2$")
plt.fill_between(wlgrid_CH4_Leff,y_meas_CH4_Leff,y_meas45,facecolor='orange',alpha=0.4,\
step='mid',label=r"CH$_4$ $\times 2$",zorder=3)
plt.fill_between(wlgrid_O2_Leff,y_meas_O2_Leff,y_meas45,facecolor='red',alpha=0.4,\
step='mid',label=r"O$_2$ $\times 2$")
plt.xlabel(r'Wavelength (' + u'\u03bc' + 'm)')
plt.ylabel(r'Flux ratio ($F_{\rm p}/F_{\rm s}$)')
plt.legend(loc='best',frameon=False,prop={'size': 12})

for tick in range(len(H2O_feat_y_R)):
    plt.text(H2O_feat_R[tick],H2O_feat_y_R[tick]*0.79,'H$_2$O',color='black',rotation='vertical')

plt.text(0.9,6.8e-10,'Low',color=c_low,fontsize=14)

#Subpplot Labels
plt.text(0.02,7.2e-10,'c',fontweight='bold',ha='center',fontsize=14)



# plt.savefig('Spectral_Plot_Full_Effects_4.png',dpi=300)
# plt.savefig('Spectral_Plot_Full_Effects_4.svg')
plt.draw()
plt.show()
plt.close()
